@@ -1,5 +1,6 @@
+#!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright (c) 2010 Sebastian Wiesner <lunaryorn@googlemail.com>
+# Copyright (c) 2010, 2011 Sebastian Wiesner <lunaryorn@googlemail.com>
 
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -33,6 +34,8 @@
 """
 
 
+from __future__ import (print_function, division, unicode_literals,
+                        absolute_import)
 
 import sys
 import os
@@ -86,7 +89,6 @@ class SourcePackage(namedtuple(
 
     def make_build(self, download_directory, target_directory):
         archive = self.download(download_directory)
-        ext = os.path.splitext(archive)[1]
         self.log.info('extracting %s to %s', archive, target_directory)
         self._check_call(['tar', '-x', '-a', '-f', archive,
                           '-C', target_directory])
@@ -153,7 +155,7 @@ class PyQtBuild(SipBuild):
 
 class CMakeBuild(Build):
 
-    configure_options = []
+    configure_options = ['-DBUILD_TESTS=OFF']
 
     @property
     def build_directory(self):
@@ -173,7 +175,7 @@ class CMakeBuild(Build):
 
 
 class PySideBuild(CMakeBuild):
-    configure_options = [
+    configure_options = CMakeBuild.configure_options + [
         '-DDISABLE_{0}=ON'.format(mod) for mod in
         ('QtGui', 'QtMultimedia', 'QtNetwork', 'QtOpenGL', 'QtScript',
          'QtScriptTools', 'QtSql', 'QtSvg', 'QtWebKit', 'QtXml',
@@ -195,32 +197,32 @@ class AutotoolsBuild(Build):
 RIVERBANK_DOWNLOAD_URL = 'http://www.riverbankcomputing.com/static/Downloads/{0}'
 
 PYQT4_SOURCES = [
-    SourcePackage('sip', '4.11.1', SipBuild,
+    SourcePackage('sip', '4.12', SipBuild,
                   RIVERBANK_DOWNLOAD_URL.format(
                       'sip4/{name}-{version}.tar.gz')),
-    SourcePackage('PyQt-x11-gpl', '4.7.7', PyQtBuild,
+    SourcePackage('PyQt-x11-gpl', '4.8.2', PyQtBuild,
                   RIVERBANK_DOWNLOAD_URL.format(
                       'PyQt4/{name}-{version}.tar.gz'))]
 
 PYSIDE_DOWNLOAD_URL = 'http://www.pyside.org/files/{0}'
 
 PYSIDE_SOURCES = [
-    SourcePackage('apiextractor', '0.8.0', CMakeBuild,
+    SourcePackage('apiextractor', '0.9.1', CMakeBuild,
                   PYSIDE_DOWNLOAD_URL.format('{name}-{version}.tar.bz2')),
-    SourcePackage('generatorrunner', '0.6.1', CMakeBuild,
+    SourcePackage('generatorrunner', '0.6.3', CMakeBuild,
                   PYSIDE_DOWNLOAD_URL.format('{name}-{version}.tar.bz2')),
-    SourcePackage('shiboken', '0.5.0', CMakeBuild,
+    SourcePackage('shiboken', '1.0.0~beta2', CMakeBuild,
                   PYSIDE_DOWNLOAD_URL.format('{name}-{version}.tar.bz2')),
-    SourcePackage('pyside', 'qt4.6+0.4.1', PySideBuild,
+    SourcePackage('pyside', 'qt4.7+1.0.0~beta2', PySideBuild,
                   PYSIDE_DOWNLOAD_URL.format('{name}-{version}.tar.bz2')),
     ]
 
 GNOME_DOWNLOAD_URL = 'http://ftp.gnome.org/pub/GNOME/sources/{0}'
 
 GOBJECT_SOURCES = [
-    SourcePackage('pygobject', '2.26.0', AutotoolsBuild,
+    SourcePackage('pygobject', '2.27.0', AutotoolsBuild,
                   GNOME_DOWNLOAD_URL.format(
-                      '{name}/2.26/{name}-{version}.tar.bz2'))]
+                      '{name}/2.27/{name}-{version}.tar.bz2'))]
 
 
 def build_all(sources, download_directory, build_directory):
@@ -228,27 +230,75 @@ def build_all(sources, download_directory, build_directory):
         source.build(download_directory, build_directory)
 
 
-def have_pyqt4_qtcore(expected_version):
+def import_string(import_name, silent=False):
+    """
+    Import an object based on a string given by ``import_name``.  An import
+    path can be specified either in dotted notation
+    (``'xml.sax.saxutils.escape'``) or with a colon as object delimiter
+    (``'xml.sax.saxutils:escape'``).
+
+    Return the imported object.  If the import or the attribute access fails
+    and ``silent`` is ``True``, ``None`` will be returned.  Otherwise
+    :exc:`~exceptions.ImportError` or :exc:`~exceptions.AttributeError` will
+    be raised.
+
+    This function was shamelessly stolen from Werkzeug_.  Thanks to Armin
+    Ronacher.
+
+    .. _werkzeug: http://werkzeug.pocoo.org
+    """
     try:
-        QtCore = __import__('PyQt4.QtCore', globals(), locals(), ['QtCore'])
+        if ':' in import_name:
+            module, obj = import_name.split(':', 1)
+        elif '.' in import_name:
+            module, obj = import_name.rsplit('.', 1)
+        else:
+            return __import__(import_name)
+        # to make things run on python 2 and python 3
+        obj = str(obj)
+        module = str(module)
+        return getattr(__import__(module, None, None, [obj]), obj)
+    except (ImportError, AttributeError):
+        if not silent:
+            raise
+
+
+def have_pyqt4_qtcore(expected_version):
+    log = logging.getLogger('pyqt4')
+    try:
+        QtCore = import_string('PyQt4.QtCore')
+        log.info('excepted version %s, actual version %s',
+                 expected_version, QtCore.PYQT_VERSION_STR)
         return QtCore.PYQT_VERSION_STR == expected_version
     except ImportError:
+        log.exception('QtCore not found')
         return False
 
 
 def have_pyside_qtcore():
+    log = logging.getLogger('pyside')
     try:
-        QtCore = __import__('PySide.QtCore', globals(), locals(), ['QtCore'])
+        import_string('PySide.QtCore')
         return True
     except ImportError:
+        log.exception('QtCore not found')
         return False
 
 
-def have_gobject():
-    for lib in ('glib', 'gobject'):
+def have_gobject(expected_version):
+    log = logging.getLogger('pygobject')
+    expected_version = tuple(map(int, expected_version.split('.')[:3]))
+    for libname in ('glib', 'gobject'):
         try:
-            __import__(lib)
+            lib = import_string(libname)
+            version_attr = 'py{0}_version'.format(libname)
+            lib_version = getattr(lib, version_attr)
+            log.info('%s: expected version %s, actual version %s',
+                     libname, expected_version, lib_version)
+            if lib_version < expected_version:
+                return False
         except ImportError:
+            log.exception('%s not found', libname)
             return False
     return True
 
@@ -274,15 +324,9 @@ def main():
     if sys.version_info[0] < 3:
         # pyside and pygobject are not available for python 3 yet
         if not have_pyside_qtcore():
-            lib_paths = os.environ.get(
-                'LD_LIBRARY_PATH', '').split(os.pathsep)
-            lib_paths.insert(0, os.path.join(sys.prefix, 'lib'))
-            lib_path = os.pathsep.join(lib_paths)
-            logging.info('setting library path to %r', lib_path)
-            os.environ['LD_LIBRARY_PATH'] = lib_path
             build_all(PYSIDE_SOURCES, download_directory, build_directory)
 
-        if not have_gobject():
+        if not have_gobject(GOBJECT_SOURCES[0].version):
             build_all(GOBJECT_SOURCES, download_directory, build_directory)
 
 
