@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2010 Sebastian Wiesner <lunaryorn@googlemail.com>
+# Copyright (C) 2010, 2011 Sebastian Wiesner <lunaryorn@googlemail.com>
 
 # This library is free software; you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published by the
@@ -27,15 +27,27 @@
     Usage
     -----
 
+    To use :mod:`pyudev`, simply import it:
+
+    >>> import pyudev
+    >>> pyudev.__version__
+    '0.8'
+    >>> pyudev.udev_version()
+    165
+
+    With the exception of the toolkit integration all classes and functions are
+    available in the top-level namespace of :mod:`pyudev`.
+
     The library context
     ^^^^^^^^^^^^^^^^^^^
 
-    First import :mod:`pyudev` and create a :class:`Context` object.  This
-    object is mandatory to use this library:
+    To use these classes and functions, a :class:`Context` object is required:
 
-    >>> import pyudev
     >>> context = pyudev.Context()
 
+    You can see this object as some kind of connection to the udev database,
+    through which the contents of this database can be queried.
+
     Listing devices
     ^^^^^^^^^^^^^^^
 
@@ -44,16 +56,14 @@
 
     >>> devices = context.list_devices()
 
-    ``devices`` is an instance of :class:`Enumerator`.  This class provides
-    some methods to filter the list of devices.  You can filter by specific
-    subsystems and by properties.  For instance, the following code only
-    matches all mouse devices:
+    You can filter the devices by specific subsystems or properties.  For
+    instance, the following code only matches all mouse devices:
 
-    >>> devices = devices.match_subsystem('input').match_property(
-    ...     'ID_INPUT_MOUSE', True)
+    >>> devices = context.list_devices(subsystem='input', ID_INPUT_MOUSE=True)
 
-    Once the (optional) filters are applied, you can iterate over
-    ``devices``.  This yields :class:`Device` objects, which provide various
+    In both cases ``devices`` is an instance of :class:`Enumerator`.  When
+    iterated over, this class yields :class:`Device` objects, representing
+    those devices, which match the filters.  :class:`Device` provide various
     attributes to access information:
 
     >>> for device in devices:
@@ -127,13 +137,35 @@
     pyudev provides classes, which integrate monitoring into the event loop
     of GUI toolkits.  Currently, PyQt4, PySide and pygobject are supported.
 
+
+    A note on versioning
+    --------------------
+
+    :mod:`pyudev` tries to provide all features of recent udev versions, while
+    maintaining compatibility with older versions.  This means, that some
+    attributes of :mod:`pyudev` classes are not available, if udev is too old.
+    Whenever this is the case, the minimum version of udev required to use the
+    attribute is described in the documentation, see for instance
+    :attr:`Device.is_initialized`.
+
+    You can use :func:`udev_version()` to check the version of udev and see, if
+    it is recent enough for your needs:
+
+    >>> if pyudev.udev_version() < 165:
+    ...     print('udev is somewhat older here')
+    ... else:
+    ...     print('udev is quite up to date here')
+    ...
+    udev is quite up to date here
+
     .. moduleauthor::  Sebastian Wiesner  <lunaryorn@googlemail.com>
 """
 
-from __future__ import absolute_import
+from __future__ import (print_function, division, unicode_literals,
+                        absolute_import)
 
 
-__version__ = '0.7'
+__version__ = '0.8'
 __all__ = ['Context', 'Device']
 
 
