@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2010 Sebastian Wiesner <lunaryorn@googlemail.com>
+# Copyright (C) 2010, 2011 Sebastian Wiesner <lunaryorn@googlemail.com>
 
 # This library is free software; you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published by the
@@ -16,25 +16,68 @@
 # Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 
-from __future__ import absolute_import
+"""
+    pyudev.core
+    ===========
+
+    Core types and function of :mod:`pyudev`.
+
+    .. moduleauthor::  Sebastian Wiesner  <lunaryorn@googlemail.com>
+"""
+
+
+from __future__ import (print_function, division, unicode_literals,
+                        absolute_import)
 
-import sys
 import os
+from subprocess import Popen, PIPE, CalledProcessError
 from itertools import count
 from collections import Mapping
+from datetime import timedelta
 
 from pyudev._libudev import libudev
-from pyudev._util import (assert_bytes, property_value_to_bytes,
-                          call_handle_error_return, udev_list_iterate,
-                          string_to_bool)
+from pyudev._util import (assert_unicode, assert_bytes, udev_list_iterate,
+                          property_value_to_bytes, string_to_bool)
 
 
 __all__ = [
-    'Context', 'Enumerator', 'Device', 'Attributes',
+    'udev_version', 'Context', 'Enumerator', 'Device', 'Attributes',
     'DeviceNotFoundError', 'DeviceNotFoundAtPathError',
     'DeviceNotFoundByNameError', 'DeviceNotFoundInEnvironmentError']
 
 
+def udev_version():
+    """
+    Get the version of the underlying udev library.
+
+    udev doesn't use a standard major-minor versioning scheme, but instead
+    labels releases with a single consecutive number.  Consequently, the
+    version number returned by this function is a single integer, and not a
+    tuple (like for instance the interpreter version in
+    :data:`sys.version_info`).
+
+    As libudev itself does not provide a function to query the version number,
+    this function calls the ``udevadm`` utilitiy, so be prepared to catch
+    :exc:`~exceptions.EnvironmentError` and
+    :exc:`~subprocess.CalledProcessError` if you call this function.
+
+    Return the version number as single integer.  Raise
+    :exc:`~exceptions.ValueError`, if the version number retrieved from udev
+    could not be converted to an integer.  Raise
+    :exc:`~exceptions.EnvironmentError`, if ``udevadm`` was not found, or could
+    not be executed.  Raise :exc:`subprocess.CalledProcessError`, if
+    ``udevadm`` returned a non-zero exit code.
+
+    .. versionadded:: 0.8
+    """
+    command = ['udevadm', '--version']
+    udevadm = Popen(command, stdout=PIPE)
+    stdout = assert_unicode(udevadm.communicate()[0]).strip()
+    if udevadm.returncode != 0:
+        raise CalledProcessError(0, command)
+    return int(stdout)
+
+
 class Context(object):
     """
     The udev context.
@@ -62,8 +105,7 @@ class Context(object):
         The mount point can be overwritten using the environment variable
         :envvar:`SYSFS_PATH`.  Use this for testing purposes.
         """
-        return libudev.udev_get_sys_path(self._context).decode(
-            sys.getfilesystemencoding())
+        return assert_unicode(libudev.udev_get_sys_path(self._context))
 
     @property
     def device_path(self):
@@ -72,18 +114,24 @@ class Context(object):
 
         This can be overridden in the udev configuration.
         """
-        return libudev.udev_get_dev_path(self._context).decode(
-            sys.getfilesystemencoding())
+        return assert_unicode(libudev.udev_get_dev_path(self._context))
 
-    def list_devices(self):
+    def list_devices(self, **kwargs):
         """
         List all available devices.
 
+        The arguments of this method are the same as for
+        :meth:`Enumerator.match()`.  In fact, the arguments are simply passed
+        straight to method :meth:`~Enumerator.match()`.
+
         This function creates and returns an :class:`Enumerator` object,
         that can be used to filter the list of devices, and eventually
         retrieve :class:`Device` objects representing matching devices.
+
+        .. versionchanged:: 0.8
+           Accept keyword arguments now for easy matching
         """
-        return Enumerator(self)
+        return Enumerator(self).match(**kwargs)
 
 
 class Enumerator(object):
@@ -128,6 +176,42 @@ class Enumerator(object):
     def __del__(self):
         libudev.udev_enumerate_unref(self._enumerator)
 
+    def match(self, **kwargs):
+        """
+        Include devices according to the rules defined by the keyword
+        arguments.  These keyword arguments are interpreted as follows:
+
+        - The value for the keyword argument ``subsystem`` is forwarded to
+          :meth:`match_subsystem()`.
+        - The value for the keyword argument ``sys_name`` is forwared to
+          :meth:`match_sys_name()`.
+        - The value for the keyword argument ``tag`` is forwared to
+          :meth:`match_tag()`
+        - All other keyword arguments are forwareded one by one to
+          :meth:`match_property()`.  The keyword argument itself is interpreted
+          as property name, the value of the keyword argument as the property
+          value.
+
+        All keyword arguments are optional, calling this method without no
+        arguments at all is simply a noop.
+
+        Return the instance again.
+
+        .. versionadded:: 0.8
+        """
+        subsystem = kwargs.pop('subsystem', None)
+        if subsystem is not None:
+            self.match_subsystem(subsystem)
+        sys_name = kwargs.pop('sys_name', None)
+        if sys_name is not None:
+            self.match_sys_name(sys_name)
+        tag = kwargs.pop('tag', None)
+        if tag is not None:
+            self.match_tag(tag)
+        for property, value in kwargs.iteritems():
+            self.match_property(property, value)
+        return self
+
     def match_subsystem(self, subsystem):
         """
         Include all devices, which are part of the given ``subsystem``.
@@ -137,11 +221,24 @@ class Enumerator(object):
 
         Return the instance again.
         """
-        call_handle_error_return(
-            libudev.udev_enumerate_add_match_subsystem,
+        libudev.udev_enumerate_add_match_subsystem(
             self._enumerator, assert_bytes(subsystem))
         return self
 
+    def match_sys_name(self, sys_name):
+        """
+        Include all devices with the given name.
+
+        ``sys_name`` is a byte or unicode string containing the device name.
+
+        Return the instance again.
+
+        .. versionadded:: 0.8
+        """
+        libudev.udev_enumerate_add_match_sysname(self._enumerator,
+                                                 assert_bytes(sys_name))
+        return self
+
     def match_property(self, property, value):
         """
         Include all devices, whose ``property`` has the given ``value``.
@@ -157,8 +254,7 @@ class Enumerator(object):
 
         Return the instance again.
         """
-        call_handle_error_return(
-            libudev.udev_enumerate_add_match_property,
+        libudev.udev_enumerate_add_match_property(
             self._enumerator, assert_bytes(property),
             property_value_to_bytes(value))
         return self
@@ -170,10 +266,34 @@ class Enumerator(object):
         ``tag`` is a byte or unicode string containing the tag name.
 
         Return the instance again.
+
+        .. versionadded:: 0.6
+
+        .. udevminversion:: 154
         """
-        call_handle_error_return(
-            libudev.udev_enumerate_add_match_tag,
-            self._enumerator, assert_bytes(tag))
+        libudev.udev_enumerate_add_match_tag(self._enumerator,
+                                             assert_bytes(tag))
+        return self
+
+    def match_is_initialized(self):
+        """
+        Include only devices, which are initialized.
+
+        Initialized devices have properly set device node permissions and
+        context, and are (in case of network devices) fully renamed.
+
+        Currently this will not affect devices which do not have device nodes
+        and are not network interfaces.
+
+        Return the instance again.
+
+        .. seealso:: :attr:`Device.is_initialized`
+
+        .. versionadded:: 0.8
+
+        .. udevminversion:: 165
+        """
+        libudev.udev_enumerate_add_match_is_initialized(self._enumerator)
         return self
 
     def match_children(self, device):
@@ -192,8 +312,7 @@ class Enumerator(object):
 
         Yield :class:`Device` objects.
         """
-        call_handle_error_return(
-            libudev.udev_enumerate_scan_devices, self._enumerator)
+        libudev.udev_enumerate_scan_devices(self._enumerator)
         entry = libudev.udev_enumerate_get_list_entry(self._enumerator)
         for name in udev_list_iterate(entry):
             device = Device.from_sys_path(self.context, name)
@@ -314,6 +433,8 @@ class Device(Mapping):
 
         Return a :class:`Device` object for the device.  Raise
         :exc:`DeviceNotFoundAtPathError`, if no device was found for ``path``.
+
+        .. versionadded:: 0.4
         """
         if not path.startswith(context.sys_path):
             path = os.path.join(context.sys_path, path.lstrip(os.sep))
@@ -351,7 +472,8 @@ class Device(Mapping):
     @classmethod
     def from_name(cls, context, subsystem, sys_name):
         """
-        Create a new device from a given ``subsystem`` and a given ``sys_name``:
+        Create a new device from a given ``subsystem`` and a given
+        ``sys_name``:
 
         >>> context = pyudev.Context()
         >>> sda = pyudev.Device.from_name(context, 'block', 'sda')
@@ -364,8 +486,10 @@ class Device(Mapping):
         denote the subsystem and the name of the device to create.
 
         Return a :class:`Device` object for the device.  Raise
-        :exc:`DeviceNotFoundByNameError`, if no device was found with the
-        given name.
+        :exc:`DeviceNotFoundByNameError`, if no device was found with the given
+        name.
+
+        .. versionadded:: 0.5
         """
         device = libudev.udev_device_new_from_subsystem_sysname(
             context._context, assert_bytes(subsystem),
@@ -390,6 +514,10 @@ class Device(Mapping):
         Return a :class:`Device` object constructed from the environment.
         Raise :exc:`DeviceNotFoundInEnvironmentError`, if no device could be
         created from the environment.
+
+        .. versionadded:: 0.6
+
+        .. udevminversion:: 152
         """
         device = libudev.udev_device_new_from_environment(context._context)
         if not device:
@@ -456,8 +584,7 @@ class Device(Mapping):
         Absolute path of this device in ``sysfs`` including the ``sysfs``
         mount point as unicode string.
         """
-        return libudev.udev_device_get_syspath(self._device).decode(
-            sys.getfilesystemencoding())
+        return assert_unicode(libudev.udev_device_get_syspath(self._device))
 
     @property
     def device_path(self):
@@ -469,34 +596,33 @@ class Device(Mapping):
         mount point.  However, the path is absolute and starts with a slash
         ``'/'``.
         """
-        return libudev.udev_device_get_devpath(self._device).decode(
-            sys.getfilesystemencoding())
+        return assert_unicode(libudev.udev_device_get_devpath(self._device))
 
     @property
     def subsystem(self):
         """
         Name of the subsystem this device is part of as unicode string.
         """
-        return libudev.udev_device_get_subsystem(self._device).decode(
-            sys.getfilesystemencoding())
+        return assert_unicode(libudev.udev_device_get_subsystem(self._device))
 
     @property
     def sys_name(self):
         """
         Device file name inside ``sysfs`` as unicode string.
         """
-        return libudev.udev_device_get_sysname(self._device).decode(
-            sys.getfilesystemencoding())
+        return assert_unicode(libudev.udev_device_get_sysname(self._device))
 
     @property
     def driver(self):
         """
         The driver name as unicode string, or ``None``, if there is no
         driver for this device.
+
+        .. versionadded:: 0.5
         """
         driver = libudev.udev_device_get_driver(self._device)
         if driver:
-            return driver.decode(sys.getfilesystemencoding())
+            return assert_unicode(driver)
 
     @property
     def device_node(self):
@@ -512,7 +638,49 @@ class Device(Mapping):
         """
         node = libudev.udev_device_get_devnode(self._device)
         if node:
-            return node.decode(sys.getfilesystemencoding())
+            return assert_unicode(node)
+
+    @property
+    def is_initialized(self):
+        """
+        ``True``, if the device is initialized, ``False`` otherwise.
+
+        A device is initialized, if udev has already handled this device and
+        has set up device node permissions and context, or renamed a network
+        device.
+
+        Consequently, this property is only implemented for devices with a
+        device node or for network devices.  On all other devices this property
+        is always ``True``.
+
+        It is *not* recommended, that you use uninitialized devices.
+
+        .. seealso:: :attr:`time_since_initialized`
+
+        .. versionadded:: 0.8
+
+        .. udevminversion:: 165
+        """
+        return bool(libudev.udev_device_get_is_initialized(self._device))
+
+    @property
+    def time_since_initialized(self):
+        """
+        The time elapsed since initialization as :class:`~datetime.timedelta`.
+
+        This property is only implemented on devices, which need to store
+        properties in the udev database.  On all other devices this property is
+        simply zero :class:`~datetime.timedelta`.
+
+        .. seealso:: :attr:`is_initialized`
+
+        .. versionadded:: 0.8
+
+        .. udevminversion:: 165
+        """
+        microseconds = libudev.udev_device_get_usec_since_initialized(
+            self._device)
+        return timedelta(microseconds=microseconds)
 
     @property
     def device_links(self):
@@ -532,7 +700,7 @@ class Device(Mapping):
         """
         entry = libudev.udev_device_get_devlinks_list_entry(self._device)
         for name in udev_list_iterate(entry):
-            yield name.decode(sys.getfilesystemencoding())
+            yield assert_unicode(name)
 
     @property
     def attributes(self):
@@ -548,6 +716,8 @@ class Device(Mapping):
 
         The values of these attributes are not always proper strings, and
         can contain arbitrary bytes.
+
+        .. versionadded:: 0.5
         """
         return self._attributes
 
@@ -555,10 +725,14 @@ class Device(Mapping):
     def tags(self):
         """
         An iterator, which yields all attached tags as unicode strings.
+
+        .. versionadded:: 0.6
+
+        .. udevminversion:: 154
         """
         entry = libudev.udev_device_get_tags_list_entry(self._device)
         for tag in udev_list_iterate(entry):
-            yield tag.decode(sys.getfilesystemencoding())
+            yield assert_unicode(tag)
 
     def __iter__(self):
         """
@@ -569,7 +743,7 @@ class Device(Mapping):
         """
         entry = libudev.udev_device_get_properties_list_entry(self._device)
         for name in udev_list_iterate(entry):
-            yield name.decode(sys.getfilesystemencoding())
+            yield assert_unicode(name)
 
     def __len__(self):
         """
@@ -596,7 +770,7 @@ class Device(Mapping):
             self._device, assert_bytes(property))
         if value is None:
             raise KeyError('No such property: {0}'.format(property))
-        return value.decode(sys.getfilesystemencoding())
+        return assert_unicode(value)
 
     def asint(self, property):
         """
@@ -684,6 +858,8 @@ class Attributes(Mapping):
     Therefore all well-known dicitionary methods and operators
     (e.g. ``.keys()``, ``.items()``, ``in``) are available to access device
     attributes.
+
+    .. versionadded:: 0.5
     """
 
     def __init__(self, device):
@@ -746,7 +922,7 @@ class Attributes(Mapping):
         for this device, or :exc:`~exceptions.UnicodeDecodeError`, if the
         content of the attribute cannot be decoded into a unicode string.
         """
-        return self[attribute].decode(sys.getfilesystemencoding())
+        return assert_unicode(self[attribute])
 
     def asint(self, attribute):
         """
