@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2010 Sebastian Wiesner <lunaryorn@googlemail.com>
+# Copyright (C) 2010, 2011 Sebastian Wiesner <lunaryorn@googlemail.com>
 
 # This library is free software; you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published by the
@@ -15,6 +15,8 @@
 # along with this library; if not, write to the Free Software Foundation,
 # Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import (print_function, division, unicode_literals,
+                        absolute_import)
 
 import sys
 import re
@@ -26,7 +28,7 @@ from contextlib import contextmanager
 from subprocess import check_call
 
 import mock
-import py.test
+import pytest
 
 import pyudev
 
@@ -171,8 +173,8 @@ def patch_libudev(funcname):
 def _need_privileges(func):
     @wraps(func)
     def wrapped(*args):
-        if not py.test.config.getvalue('allow_privileges'):
-            py.test.skip('privileges disabled')
+        if not pytest.config.getvalue('allow_privileges'):
+            pytest.skip('privileges disabled')
         return func(*args)
     return wrapped
 
@@ -206,10 +208,29 @@ def is_unicode_string(value):
     return isinstance(value, unicode_type)
 
 
+def check_udev_version(version_spec):
+    """
+    Decorator to check the udev version.
+
+    ``version_spec`` is a string containing a comparison operator and the right
+    hand sight operand of this comparison, like ``'>= 165'``.  The left hand
+    side is automatically provided by the current udev version.
+
+    If the udev version matches the spec, the test is run.  Otherwise the test
+    is skipped.
+    """
+    expr_template = 'not ({0} {1})'
+    actual_version = pyudev.udev_version()
+    expr = expr_template.format(actual_version, version_spec)
+    reason = 'udev version mismatch: {0} required, {1} found'.format(
+        version_spec, actual_version)
+    return pytest.mark.skipif(str(expr), reason=reason)
+
+
 def pytest_namespace():
     return dict((func.__name__, func) for func in
                 (get_device_sample, patch_libudev, load_dummy,
-                 unload_dummy, is_unicode_string))
+                 unload_dummy, is_unicode_string, check_udev_version))
 
 
 def pytest_addoption(parser):
@@ -242,6 +263,7 @@ def pytest_configure(config):
         ['power_on_acct', 'temp1_input', 'charge_now', 'current_now',
          'urbnum'])
     config.udev_database = _read_udev_database(config.properties_blacklist)
+    config.udev_version = pyudev.udev_version()
 
 
 def pytest_funcarg__database(request):
