@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2010 Sebastian Wiesner <lunaryorn@googlemail.com>
+# Copyright (C) 2010, 2011 Sebastian Wiesner <lunaryorn@googlemail.com>
 
 # This library is free software; you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published by the
@@ -16,6 +16,12 @@
 # Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 
+from __future__ import (print_function, division, unicode_literals,
+                        absolute_import)
+
+import pytest
+
+
 def test_match_subsystem(context):
     devices = context.list_devices().match_subsystem('input')
     for n, device in enumerate(devices, start=1):
@@ -23,6 +29,13 @@ def test_match_subsystem(context):
     assert n > 0
 
 
+def test_match_sys_name(context):
+    devices = context.list_devices().match_sys_name('sda')
+    for n, device in enumerate(devices, start=1):
+        assert device.sys_name == 'sda'
+    assert n > 0
+
+
 def test_match_property_string(context):
     devices = context.list_devices().match_property('DRIVER', 'usb')
     for n, device in enumerate(devices, start=1):
@@ -44,10 +57,19 @@ def test_match_property_bool(context):
     assert n > 0
 
 
+@pytest.check_udev_version('>= 154')
 def test_match_tags(context):
     raise NotImplementedError()
 
 
+@pytest.check_udev_version('>= 165')
+def test_match_is_initialized(context):
+    funcname = 'udev_enumerate_add_match_is_initialized'
+    with pytest.patch_libudev(funcname) as func:
+        context.list_devices().match_is_initialized()
+        assert func.called
+
+
 def test_combined_matches_of_same_type(context):
     """
     Test for behaviour as observed in #1
@@ -68,3 +90,24 @@ def test_combined_matches_of_different_types(context):
         devices.match_property(property, 'disk')
     devices = list(devices)
     assert not devices
+
+
+def test_match(context):
+    devices = context.list_devices().match(
+        subsystem='input', ID_INPUT_MOUSE=True, sys_name='mouse0')
+    for n, device in enumerate(devices, start=1):
+        assert device.subsystem == 'input'
+        assert device['ID_INPUT_MOUSE'] == '1'
+        assert device.sys_name == 'mouse0'
+    assert n > 0
+
+
+@pytest.mark.match
+def test_list_devices(context):
+    devices = context.list_devices(subsystem='input', ID_INPUT_MOUSE=True,
+                                   sys_name='mouse0')
+    for n, device in enumerate(devices, start=1):
+        assert device.subsystem == 'input'
+        assert device['ID_INPUT_MOUSE'] == '1'
+        assert device.sys_name == 'mouse0'
+    assert n > 0
