@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2010 Sebastian Wiesner <lunaryorn@googlemail.com>
+# Copyright (C) 2010, 2011 Sebastian Wiesner <lunaryorn@googlemail.com>
 
 # This library is free software; you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published by the
@@ -15,40 +15,17 @@
 # along with this library; if not, write to the Free Software Foundation,
 # Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
-from __future__ import unicode_literals
+from __future__ import (print_function, division, unicode_literals,
+                        absolute_import)
 
 import sys
-import os
-import errno
 
-import py.test
-from mock import Mock
+import pytest
 
 from pyudev import _util
 
 
-def test_call_handle_error_return_no_error():
-    func = Mock(return_value=0)
-    _util.call_handle_error_return(func, 'spam', 'eggs')
-    assert func.called_with_args('spam', 'eggs')
-
-def test_call_handle_error_return_memory_error():
-    func = Mock(return_value=-errno.ENOMEM)
-    with py.test.raises(MemoryError):
-        _util.call_handle_error_return(func, 'spam', 'eggs')
-    assert func.called_with_args('spam', 'eggs')
-
-def test_call_handle_error_return_environment_error():
-    func = Mock(return_value=-errno.ENOENT)
-    with py.test.raises(EnvironmentError) as exc_info:
-        _util.call_handle_error_return(func, 'spam', 'eggs')
-    error = exc_info.value
-    assert error.errno == errno.ENOENT
-    assert error.strerror == os.strerror(errno.ENOENT)
-    assert func.called_with_args('spam', 'eggs')
-
-
-@py.test.mark.conversion
+@pytest.mark.conversion
 def test_assert_bytes():
     assert isinstance(_util.assert_bytes('hello world'), bytes)
     assert _util.assert_bytes('hello world') == b'hello world'
@@ -56,13 +33,27 @@ def test_assert_bytes():
     assert _util.assert_bytes(hello) is hello
 
 
-@py.test.mark.conversion
+@pytest.mark.conversion
 def test_assert_bytes_none():
-    with py.test.raises(AttributeError):
+    with pytest.raises(AttributeError):
         _util.assert_bytes(None)
 
 
-@py.test.mark.conversion
+@pytest.mark.conversion
+def test_assert_unicode():
+    assert pytest.is_unicode_string(_util.assert_unicode(b'hello world'))
+    assert _util.assert_unicode(b'hello world') == 'hello world'
+    hello = 'hello world'
+    assert _util.assert_unicode(hello) is hello
+
+
+@pytest.mark.conversion
+def test_assert_unicode_none():
+    with pytest.raises(AttributeError):
+        _util.assert_unicode(None)
+
+
+@pytest.mark.conversion
 def test_property_value_to_bytes_string():
     hello = 'hello world'.encode(sys.getfilesystemencoding())
     assert _util.property_value_to_bytes(hello) is hello
@@ -70,12 +61,12 @@ def test_property_value_to_bytes_string():
     assert _util.property_value_to_bytes('hello world') == hello
 
 
-@py.test.mark.conversion
+@pytest.mark.conversion
 def test_property_value_to_bytes_int():
     assert _util.property_value_to_bytes(10000) == b'10000'
     assert isinstance(_util.property_value_to_bytes(10000), bytes)
 
-@py.test.mark.conversion
+@pytest.mark.conversion
 def test_property_value_to_bytes_bool():
     assert _util.property_value_to_bytes(True) == b'1'
     assert isinstance(_util.property_value_to_bytes(True), bytes)
@@ -83,18 +74,18 @@ def test_property_value_to_bytes_bool():
     assert isinstance(_util.property_value_to_bytes(False), bytes)
 
 
-@py.test.mark.conversion
+@pytest.mark.conversion
 def test_string_to_bool_true():
     assert isinstance(_util.string_to_bool('1'), bool)
     assert _util.string_to_bool('1')
 
-@py.test.mark.conversion
+@pytest.mark.conversion
 def test_string_to_bool_false():
     assert isinstance(_util.string_to_bool('0'), bool)
     assert not _util.string_to_bool('0')
 
-@py.test.mark.conversion
+@pytest.mark.conversion
 def test_string_to_bool_invalid_value():
-    with py.test.raises(ValueError) as exc_info:
+    with pytest.raises(ValueError) as exc_info:
         _util.string_to_bool('foo')
     assert str(exc_info.value) == 'Not a boolean value: {0!r}'.format('foo')
